@@ -50,26 +50,8 @@ acb_init(acb_t x)
 
 void acb_clear(acb_t x);
 
-ACB_INLINE acb_ptr
-_acb_vec_init(slong n)
-{
-    slong i;
-    acb_ptr v = (acb_ptr) flint_malloc(sizeof(acb_struct) * n);
-
-    for (i = 0; i < n; i++)
-        acb_init(v + i);
-
-    return v;
-}
-
-ACB_INLINE void
-_acb_vec_clear(acb_ptr v, slong n)
-{
-    slong i;
-    for (i = 0; i < n; i++)
-        acb_clear(v + i);
-    flint_free(v);
-}
+acb_ptr _acb_vec_init(slong n);
+void _acb_vec_clear(acb_ptr v, slong n);
 
 ACB_INLINE arb_ptr acb_real_ptr(acb_t z) { return acb_realref(z); }
 ACB_INLINE arb_ptr acb_imag_ptr(acb_t z) { return acb_imagref(z); }
@@ -110,6 +92,12 @@ acb_is_int(const acb_t z)
     return arb_is_zero(acb_imagref(z)) && arb_is_int(acb_realref(z));
 }
 
+ACB_INLINE int
+acb_is_int_2exp_si(const acb_t z, slong e)
+{
+    return arb_is_zero(acb_imagref(z)) && arb_is_int_2exp_si(acb_realref(z), e);
+}
+
 ACB_INLINE void
 acb_zero(acb_t z)
 {
@@ -403,12 +391,33 @@ acb_get_rad_ubound_arf(arf_t u, const acb_t z, slong prec)
     arf_mul_2exp_si(u, u, 1);
 }
 
+ACB_INLINE void
+acb_union(acb_t res, const acb_t x, const acb_t y, slong prec)
+{
+    arb_union(acb_realref(res), acb_realref(x), acb_realref(y), prec);
+    arb_union(acb_imagref(res), acb_imagref(x), acb_imagref(y), prec);
+}
+
 void acb_arg(arb_t r, const acb_t z, slong prec);
 
 void acb_sgn(acb_t res, const acb_t z, slong prec);
 
 void acb_csgn(arb_t res, const acb_t z);
 
+void acb_real_abs(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_real_sgn(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_real_heaviside(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_real_floor(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_real_ceil(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_real_max(acb_t res, const acb_t x, const acb_t y, int analytic, slong prec);
+void acb_real_min(acb_t res, const acb_t x, const acb_t y, int analytic, slong prec);
+void acb_real_sqrtpos(acb_t res, const acb_t z, int analytic, slong prec);
+
+void acb_sqrt_analytic(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_rsqrt_analytic(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_log_analytic(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_pow_analytic(acb_t res, const acb_t z, const acb_t w, int analytic, slong prec);
+
 ACB_INLINE void
 acb_add(acb_t z, const acb_t x, const acb_t y, slong prec)
 {
@@ -635,6 +644,16 @@ acb_submul_arb(acb_t z, const acb_t x, const arb_t y, slong prec)
     arb_submul(acb_imagref(z), acb_imagref(x), y, prec);
 }
 
+void acb_dot_simple(acb_t res, const acb_t initial, int subtract,
+    acb_srcptr x, slong xstep, acb_srcptr y, slong ystep, slong len, slong prec);
+void acb_dot_precise(acb_t res, const acb_t initial, int subtract,
+    acb_srcptr x, slong xstep, acb_srcptr y, slong ystep, slong len, slong prec);
+void acb_dot(acb_t res, const acb_t initial, int subtract,
+    acb_srcptr x, slong xstep, acb_srcptr y, slong ystep, slong len, slong prec);
+
+void acb_approx_dot(acb_t res, const acb_t initial, int subtract,
+    acb_srcptr x, slong xstep, acb_srcptr y, slong ystep, slong len, slong prec);
+
 void acb_inv(acb_t z, const acb_t x, slong prec);
 
 void acb_div(acb_t z, const acb_t x, const acb_t y, slong prec);
@@ -685,6 +704,7 @@ void acb_log1p(acb_t r, const acb_t z, slong prec);
 void acb_exp(acb_t r, const acb_t z, slong prec);
 void acb_exp_pi_i(acb_t r, const acb_t z, slong prec);
 void acb_exp_invexp(acb_t r, acb_t s, const acb_t z, slong prec);
+void acb_expm1(acb_t r, const acb_t z, slong prec);
 
 void acb_sin(acb_t r, const acb_t z, slong prec);
 void acb_cos(acb_t r, const acb_t z, slong prec);
@@ -738,6 +758,24 @@ acb_coth(acb_t y, const acb_t x, slong prec)
     acb_mul_onei(y, y);
 }
 
+void acb_sech(acb_t r, const acb_t z, slong prec);
+void acb_csch(acb_t r, const acb_t z, slong prec);
+
+ACB_INLINE void
+acb_sec(acb_t y, const acb_t x, slong prec)
+{
+    acb_mul_onei(y, x);
+    acb_sech(y, y, prec);
+}
+
+ACB_INLINE void
+acb_csc(acb_t y, const acb_t x, slong prec)
+{
+    acb_mul_onei(y, x);
+    acb_csch(y, y, prec);
+    acb_mul_onei(y, y);
+}
+
 void acb_sin_pi(acb_t r, const acb_t z, slong prec);
 void acb_cos_pi(acb_t r, const acb_t z, slong prec);
 void acb_sin_cos_pi(acb_t s, acb_t c, const acb_t z, slong prec);
@@ -745,6 +783,7 @@ void acb_tan_pi(acb_t r, const acb_t z, slong prec);
 void acb_cot_pi(acb_t r, const acb_t z, slong prec);
 
 void acb_sinc(acb_t res, const acb_t z, slong prec);
+void acb_sinc_pi(acb_t res, const acb_t z, slong prec);
 
 void acb_pow_arb(acb_t z, const acb_t x, const arb_t y, slong prec);
 void acb_pow(acb_t r, const acb_t x, const acb_t y, slong prec);
@@ -794,6 +833,14 @@ void acb_polylog_si(acb_t w, slong s, const acb_t z, slong prec);
 void acb_agm1(acb_t m, const acb_t z, slong prec);
 void acb_agm1_cpx(acb_ptr m, const acb_t z, slong len, slong prec);
 
+#define ACB_LAMBERTW_LEFT 2
+#define ACB_LAMBERTW_MIDDLE 4
+
+void acb_lambertw_asymp(acb_t res, const acb_t z, const fmpz_t k, slong L, slong M, slong prec);
+int acb_lambertw_check_branch(const acb_t w, const fmpz_t k, slong prec);
+void acb_lambertw_bound_deriv(mag_t res, const acb_t z, const acb_t ez1, const fmpz_t k);
+void acb_lambertw(acb_t res, const acb_t z, const fmpz_t k, int flags, slong prec);
+
 ACB_INLINE void
 acb_sqr(acb_t res, const acb_t val, slong prec)
 {
@@ -1137,7 +1184,7 @@ void _acb_vec_sort_pretty(acb_ptr vec, slong len);
 
 /* roots of unity */
 void acb_unit_root(acb_t res, ulong order, slong prec);
-void _acb_vec_unit_roots(acb_ptr z, slong len, slong prec);
+void _acb_vec_unit_roots(acb_ptr z, slong order, slong len, slong prec);
 
 ACB_INLINE slong
 acb_allocated_bytes(const acb_t x)
